import { Component, OnInit,ViewChild , ElementRef, TemplateRef,} from '@angular/core';
import { AllPaidTransactionsService } from '../services/all-paid-transactions.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
@Component({
  selector: 'app-all-paid-transactions',
  templateUrl: './all-paid-transactions.component.html',
  styleUrls: ['./all-paid-transactions.component.css'],
})
export class AllPaidTransactionsComponent implements OnInit {
  countryList: any = [];
  selectedCountryList: any;
  agentList: any = [];
  selectedAgentList: any = 0;
  agentBranchList: any = [];
  selectedAgentBranchList: any = 'ALL';
  // fromDate: string;
  // toDate: string;
  fromDate: any = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  toDate: any = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  gridData: any = [];
  totalPayOutAmt: any = '';
  isLoading: boolean = false;
  receiptData: any = {};
  constructor(private agentService: AllPaidTransactionsService, private modalService: BsModalService,) {}
  @ViewChild('viewTransactionReceipt') viewTransactionReceiptModal;
  bsTransactionReceiptModalRef: BsModalRef;
  ngOnInit(): void {
    this.getCountryList();
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  getCountryList() {
    this.agentService.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountryList = res?.[0]?.CountryID;
        this.onCountryChange(this.selectedCountryList);
      },
      error: (error) => {
        console.error(error);
      },
    });
  }

  onCountryChange(countryId: number) {
    if (countryId !== 0) {
      this.agentService.getAgentList(countryId).subscribe({
        next: (res: any) => {
          this.agentList = res;
        },
        error: (error) => {
          console.error(error);
        },
      });
    }
  }

  onAgentChange(agentId: number) {
    if (agentId !== 0) {
      this.agentService.getAgentBranchList(agentId).subscribe({
        next: (res: any) => {
          this.agentBranchList = res;
        },
        error: (error) => {
          console.error(error);
        },
      });
    }
  }

  displayGridData() {
    if (
      !this.fromDate ||
      !this.toDate ||
      !this.selectedCountryList ||
      // !this.selectedAgentList ||
      !this.selectedAgentBranchList
    ) {
      alert('Please select From Date, To Date, Country, Agent, and Branch.');
      return;
    }
    this.isLoading = true;
    this.agentService
      .getGridData(
        this.selectedAgentList,
        this.selectedCountryList,
        this.selectedAgentBranchList,
        this.fromDate,
        this.toDate
      )
      .subscribe({
        next: (res: any) => {
          this.isLoading=false
          if (!res || res.length === 0) {
            alert('No data found.');
            this.gridData = [];
            return;
          }
          this.gridData = res;
          this.totalPayOutAmt = res?.[0]?.Tot_PayOutAmt;
          this.isLoading = false;
        },
        error: (error) => {
          console.error(error);
          this.isLoading = false;
        },
      });
  }

  get agent() {
    let agentName = '--All--';
    const agent = this.agentList.find(
      (a) => a.Agent_ID == this.selectedAgentList
    )?.Agent_Name;
    if (agent) {
      agentName = agent;
    }
    return agentName;
  }

  getTransactionNumberDetail(item) {
    const trnNo = item?.SRTRNo;
    this.agentService.getTransactionReport(trnNo).subscribe(
      (res: any) => {
        if (res?.[0]?.IsDuplicate == true) {
          alert('No data found.');
          return;
        }
        this.receiptData = res?.[0];
        this.openTransactionReceiptModal(this.viewTransactionReceiptModal);
      }
    );
  }
  openTransactionReceiptModal(template: TemplateRef<any>) {
    this.bsTransactionReceiptModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }

}
