import { Component, OnInit } from '@angular/core';
import { AgentExchangeEarningReportService } from '../services/agent-exchange-earning-report.service';

@Component({
  selector: 'app-commission-details',
  templateUrl: './commission-details.component.html',
  styleUrls: ['./commission-details.component.css'],
})
export class CommissionDetailsComponent implements OnInit {
  countries: any[] = [];
  reportTypes: any[] = [];
  agents: any[] = [];
  currencies: any[] = [];
  masterAgents: any[] = [];
  subAgents: any[] = [];
  selectedCountryId: number = 0;
  selectedAgentId: number = 0;
  selectedSubAgentId: number = 0;
  selectedCurrencyId: number = 0;
  startDate: string = '';
  endDate: string = '';

  constructor(private reportService: AgentExchangeEarningReportService) {}

  ngOnInit(): void {
    this.loadReportTypes();
    this.loadCountries();

    const today = new Date();
    this.startDate = this.formatDate(today);
    this.endDate = this.formatDate(today);
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  loadCountries(): void {
    const userId = 0;
    this.reportService.getCountries(userId).subscribe(
      (data: any[]) => {
        this.countries = data;
      },
      (error: any) => {
        console.error('Error fetching countries:', error);
      }
    );
  }

  loadReportTypes(): void {
    this.reportService.getReportTypes().subscribe(
      (data: any[]) => {
        this.reportTypes = data;
      },
      (error: any) => {
        console.error('Error fetching report types:', error);
      }
    );
  }

  loadAgents(agentId: number, countryId: number): void {
    this.reportService.getAgents(agentId, countryId).subscribe(
      (data: any[]) => {
        this.agents = data.map((agent) => ({
          Agent_ID: agent.Agent_ID,
          Agent_Name: agent.Agent_Name,
        }));
        this.selectedAgentId = this.agents?.[0]?.Agent_ID;
        this.loadSubAgents(this.selectedAgentId, -1);
        this.loadCurrencies(this.selectedAgentId);
      },
      (error: any) => {
        console.error('Error fetching agents:', error);
      }
    );
  }

  loadSubAgents(agentId: number, sendBranch: number): void {
    this.reportService.getAgentBranches(agentId, sendBranch).subscribe(
      (data: any[]) => {
        this.subAgents = data.map((agent) => ({
          AgentBranch_ID: agent.AgentBranch_ID,
          AgentBranch_Name: agent.AgentBranch_Name,
        }));
      },
      (error: any) => {
        console.error('Error fetching sub agents:', error);
      }
    );
  }

  onCountryChange(countryId: number): void {
    const agentId = 0; // Assuming you have the agentId
    this.selectedCountryId = countryId;
    this.loadAgents(agentId, countryId);
  }

  onMasterAgentChange(agentId: number): void {
    this.selectedAgentId = agentId;
    this.loadSubAgents(this.selectedAgentId, -1);
    this.loadCurrencies(this.selectedAgentId);
  }

  loadCurrencies(agentId: number): void {
    this.reportService.getCurrencies(agentId).subscribe(
      (data: any[]) => {
        this.currencies = data.map((currency) => ({
          Currency_id: currency.Currency_id,
          CurrencyShortName: currency.CurrencyShortName,
        }));
        this.selectedCurrencyId = this.currencies?.[0]?.Currency_id;
      },
      (error: any) => {
        console.error('Error fetching currencies:', error);
      }
    );
  }

  private formatDate(date: Date): string {
    const d = new Date(date);
    let month = '' + (d.getMonth() + 1);
    let day = '' + d.getDate();
    const year = d.getFullYear();

    if (month.length < 2) month = '0' + month;
    if (day.length < 2) day = '0' + day;

    return [year, month, day].join('-');
  }
}
