import { Component, ElementRef, ViewChild } from '@angular/core';
import * as XLSX from 'xlsx';
import { ComplianceReleasedTransactionReportService } from '../services/compliance-released-transaction-report.service';

@Component({
  selector: 'app-compliance-rejected-transaction-report',
  templateUrl: './compliance-rejected-transaction-report.component.html',
  styleUrls: ['./compliance-rejected-transaction-report.component.css'],
})
export class ComplianceRejectedTransactionReportComponent {
  @ViewChild('complianceTransactionTable', { static: false }) complianceTransactionTable: ElementRef;

  transactionNo: any = '';
  fromDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  toDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  allComplianceTransactions: any = [];
  selectedFileFormat: any = '1';

  constructor(private complianceTransactionSvc: ComplianceReleasedTransactionReportService) {}

  ngOnInit(): void {}

  filterByTransactioNo() {
    if (!this.transactionNo) {
      alert('Please Enter Transaction No!');
      return;
    }

    this.complianceTransactionSvc.getDataByTransactionNo(this.transactionNo).subscribe({
      next: (res: any) => {
        this.allComplianceTransactions = res;
        this.transformData();
        if (this.allComplianceTransactions.length === 0) {
          alert('No data available');
        }
      },
      error: (err) => {
        this.allComplianceTransactions = [];
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {},
    });
  }

  displayByDate() {
    if (!this.fromDate || !this.toDate) {
      alert('Please Select From & To Date!');
      return;
    }

    this.complianceTransactionSvc.getFillGrid(this.fromDate, this.toDate).subscribe({
      next: (res: any) => {
        this.allComplianceTransactions = res;
        this.transformData();
        if (this.allComplianceTransactions.length === 0) {
          alert('No data available');
        }
      },
      error: (err) => {
        this.allComplianceTransactions = [];
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {},
    });
  }

  exportToExcel() {
    if (this.allComplianceTransactions.length === 0) {
      alert('No data available to export');
      return;
    }

    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(this.complianceTransactionTable.nativeElement);

    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    const fileName = `Compliance_Reject_Transaction_Report_${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  transformData() {
    this.allComplianceTransactions = this.allComplianceTransactions.map((item) => {
      const newItem = {};
      for (const key in item) {
        if (item.hasOwnProperty(key)) {
          const newKey = key.replace(/^\*|(?<=,)\*|\*(?=,|$)|\s+/g, ''); // Remove asterisk and replace spaces with underscores
          newItem[newKey] = item[key];
        }
      }
      return newItem;
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
