import { Component } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { InstantCashTransactionsService } from '../services/instant-cash-transactions.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-instant-cash-transactions',
  templateUrl: './instant-cash-transactions.component.html',
  styleUrls: ['./instant-cash-transactions.component.css'],
})
export class InstantCashTransactionsComponent {
  instantCashDetails: any[] = [];
  ictcNumber: string = '';
  showTable: boolean = false;
  form: FormGroup;
  isLoading: boolean = false;
  selectedFileFormat: any = '1';
  constructor(
    private fb: FormBuilder,
    private instantCashService: InstantCashTransactionsService
  ) {
    const StcurrentDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);

    const currentDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);
    this.form = this.fb.group({
      startDate: [StcurrentDate, Validators.required],
      endDate: [currentDate, Validators.required],
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  fetchData(): void {
    const { startDate, endDate } = this.form.value;

    this.isLoading = true; // Start loading
    this.instantCashService.getInstantCashDetails(startDate, endDate,this.ictcNumber).subscribe(
      (data: any[]) => {
        this.instantCashDetails = data;
        this.showTable = true;
        this.isLoading = false; // Stop loading
      },
      (error: any) => {
        console.error('Error fetching instant cash details:', error);
        this.isLoading = false; // Stop loading
      }
    );
  }

  onSubmit(): void {
    if (this.form.valid) {
      this.fetchData();
    } else {
      this.form.markAllAsTouched();
    }
  }

  get startDate() {
    return this.form.get('startDate');
  }

  get endDate() {
    return this.form.get('endDate');
  }
  
onInputChange(value: string) {
    this.ictcNumber = value; // Update the variable when input changes
  }

  exportToExcel(): void {
    let ws: XLSX.WorkSheet;

    // Convert data to worksheet
    ws = XLSX.utils.json_to_sheet(this.instantCashDetails);

    // Create a workbook and add the worksheet
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    // Determine file extension and save to file
    const fileExtension = this.selectedFileFormat === '1' ? 'xlsx' : 'csv';
    const fileName = `InstantCash_Report_${new Date().getTime()}.${fileExtension}`;

    // Save the workbook
    XLSX.writeFile(wb, fileName);
  }
}
