import { Component, TemplateRef, ViewChild } from '@angular/core';
import { ManageRemitterOccupationService } from '../services/manage-remitter-occupation.service';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ToastrService } from 'ngx-toastr';
@Component({
  selector: 'app-manage-remitter-occupation',
  templateUrl: './manage-remitter-occupation.component.html',
  styleUrls: ['./manage-remitter-occupation.component.css'],
})
export class ManageRemitterOccupationComponent {
  @ViewChild('manageOccupation') manageOccupationModal;
  bsModalRef: BsModalRef;

  occupationList: any = [];
  selectedOccupation: any = '0';
  remitterOccupation: any = [];
  isLoading: boolean = false;

  type: any = 'add';

  manageOccupationForm = new FormGroup({
    OccupationID: new FormControl(),
    RemitterOccupationName: new FormControl('', [
      Validators.required,
      Validators.pattern(/^[A-Za-z ]+$/)  // Allows only letters and spaces
    ]),
    IsActive: new FormControl(false),
  });

  constructor(
    private remitterOccupationSvc: ManageRemitterOccupationService,
    private modalService: BsModalService,
    private toastrSvc: ToastrService
  ) { }

  ngOnInit(): void {
    this.getOccupationList();
    this.getAllRecord();
  }

  getOccupationList() {
    this.isLoading = true;

    this.remitterOccupationSvc.getOccupationList().subscribe({
      next: (res: any) => {
        this.occupationList = res;
      },
      error: (err) => {
        this.occupationList = [];
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  getAllRecord(occupationName: any = 0) {
    this.isLoading = true;

    this.remitterOccupationSvc.getAllRecord(occupationName).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.remitterOccupation = res;
      },
      error: (err) => {
        this.remitterOccupation = [];
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  filter() {
    if (this.selectedOccupation) {
      this.getAllRecord(this.selectedOccupation);
    }
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openOccupationModal(type: any) {
    if (type == 'add') {
      this.manageOccupationForm.reset();
    }
    this.type = type;
    this.openModal(this.manageOccupationModal);
  }

  get f() {
    return this.manageOccupationForm.value;
  }

  editOccupation(occupation: any) {
    this.manageOccupationForm.patchValue({
      OccupationID: occupation?.OccupationID,
      RemitterOccupationName: occupation?.Occupation,
      IsActive: occupation?.IsActive,
    });
    this.openOccupationModal('edit');
  }

  addRemitterOccupation() {
    if (!this.manageOccupationForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    const occupationName = this.f.RemitterOccupationName;
    const isActive = this.f.IsActive || false;
    const occupationId = this.f.OccupationID;

    const handleResponse = (message: any) => {
      this.modalService.hide();
      this.getAllRecord();
      this.getOccupationList();
      // alert(message);
      this.toastrSvc.success(message);

    };

    const handleError = (err: any) => {
      this.modalService.hide();
      this.toastrSvc.error(err?.error?.message)
      // alert(err?.error?.message || 'Something went wrong');
    };

    const request$ =
      this.type === 'edit'
        ? this.remitterOccupationSvc.updateRemitterOccupation(
          occupationId,
          occupationName,
          isActive
        )
        : this.remitterOccupationSvc.addRemitterOccupation(
          occupationName,
          isActive
        );

    request$.subscribe({
      next: (response) => {
        handleResponse(
          response
        );
      },
      error: (err) => {
        handleError(err);
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  transformToUpperCase(event: Event, controlName: string): void {
    const input = event.target as HTMLInputElement;
    input.value = input.value.toUpperCase();
    this.manageOccupationForm.get(controlName)?.setValue(input.value);
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
