import { Component, OnInit } from '@angular/core';
import { RejectBeneficiaryComplianceReleaseService } from '../services/reject-beneficiary-compliance-release.service';
import { Router } from '@angular/router';

@Component({
  selector: 'app-reject-beneficiary-compliance-release',
  templateUrl: './reject-beneficiary-compliance-release.component.html',
  styleUrls: ['./reject-beneficiary-compliance-release.component.css'],
})
export class RejectBeneficiaryComplianceReleaseComponent implements OnInit {
  beneficiaryDetails: any[] = [];
  beneficiaryNumber: string = '';
  isLoading: boolean = false;
  noDataAvailable: boolean = false; // Flag to show "No data available" message

  constructor(
    private rejectService: RejectBeneficiaryComplianceReleaseService,
    private router: Router
  ) {}

  ngOnInit() {
    this.getStatusRecords();
  }

  navigateToBeneficiary(beneficiaryNumber: any) {
    this.router.navigate([
      `/panel/reject-beneficiary-compliance-report-reject/${beneficiaryNumber}`,
    ]);
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  onBeneficiaryNumberChange(value: string) {
    this.beneficiaryNumber = value;
  }

  searchBeneficiary() {
    if (this.beneficiaryNumber.trim() !== '') {
      this.isLoading = true;
      this.noDataAvailable = false; // Reset noDataAvailable flag

      this.rejectService.getAllRecords(this.beneficiaryNumber).subscribe(
        (response: any) => {
          this.beneficiaryDetails = response;
          this.isLoading = false;
          if (this.beneficiaryDetails.length === 0) {
            this.noDataAvailable = true; // Set flag to show "No data available"
          }
        },
        (error) => {
          console.error('Error fetching beneficiary details:', error);
          this.beneficiaryDetails = [];
          this.isLoading = false;
          this.noDataAvailable = true; // Set flag to show "No data available" on error
        }
      );
    } else {
      console.log('Please enter a beneficiary number');
      this.beneficiaryDetails = [];
      this.noDataAvailable = false; // Reset noDataAvailable flag
    }
  }

  getStatusRecords() {
    this.isLoading = true;
    this.rejectService.getStatusRecords('Release').subscribe(
      (response: any[]) => {
        this.beneficiaryDetails = response;
        this.isLoading = false;
      },
      (error) => {
        console.error('Error fetching beneficiary details:', error);
        this.isLoading = false;
      }
    );
  }

  getOfacLogDetails(transactionNumber: string) {
    this.rejectService.complicanceOfacLogDetails(transactionNumber).subscribe(
      (response: any) => {
        console.log('OFAC Log Details:', response);
      },
      (error) => {
        console.error('Error fetching OFAC Log Details:', error);
      }
    );
  }
}
