import { Component } from '@angular/core';
import { RemittancePaidUnpaidService } from '../services/remittance-paid-unpaid.service';

@Component({
  selector: 'app-remittance-paid-unpaid',
  templateUrl: './remittance-paid-unpaid.component.html',
  styleUrls: ['./remittance-paid-unpaid.component.css'],
})
export class RemittancePaidUnpaidComponent {
  fromDate: any;
  toDate: any;
  countryType: any = [
    {
      id: 'Send',
      value: 'Send',
    },
    {
      id: 'Receive',
      value: 'Receive',
    },
  ];
  selectedCountryType: any = '0';
  countries: any[];
  selectedCountry: any;
  agents: any = [];
  selectedAgent: any;
  status: any[];
  selectedStatus: any;
  transferType: any = [];
  selectedTransferType: any;
  remittancePaidUnpaid: any = [];
  totalAmount: number = 0;

  constructor(
    private remittancePaidUnPaidService: RemittancePaidUnpaidService
  ) {}

  ngOnInit(): void {
    this.loadCountries('0');
    this.loadStatus();
    this.loadTransferType();
  }

  loadCountries(countryType: string) {
    this.remittancePaidUnPaidService
      .getCountries(countryType)
      .subscribe((countries) => {
        this.countries = countries;
        this.selectedCountry = this.countries?.[0]?.CountryID;
        this.loadAgents(this.selectedCountry);
      });
  }

  onCountryChange(countryId: number) {
    this.selectedCountry = countryId;
    this.loadAgents(countryId);
  }

  loadAgents(countryId: number) {
    this.remittancePaidUnPaidService
      .getAgents(countryId)
      .subscribe((agents) => {
        this.agents = agents;
        this.selectedAgent = this.agents?.[0]?.Agent_ID;
      });
  }

  onCountryTypeChange(countryType: string) {
    if (countryType != '0') {
      this.selectedCountryType = countryType;
      this.loadCountries(countryType);
    }
  }

  loadStatus() {
    this.remittancePaidUnPaidService.getStatus().subscribe((status) => {
      this.status = status;
    });
  }

  loadTransferType() {
    this.remittancePaidUnPaidService
      .getTransferType()
      .subscribe((transferType) => {
        this.transferType = transferType;
        this.transferType = transferType.map(status => ({
          ...status,
          checked: true // default value
        }));
      });
  }

  viewReport() {
    if (!this.fromDate) {
      alert('Please Select From Date.');
      return;
    }

    if (!this.toDate) {
      alert('Please Select To Date.');
      return;
    }

    if (!this.selectedCountryType || this.selectedCountryType == '0') {
      alert('Please Select Country Type First.');
      return;
    }

    const isCheckStatus = this.status.filter((s) => s.checked == true);
    const isCheckTransferType = this.transferType.filter(
      (t) => t.checked == true
    );
    if (isCheckStatus.length == 0) {
      alert('Please Select Atleast One Status.');
      return;
    }

    if (isCheckTransferType.length == 0) {
      alert('Please Select Transfer Type.');
      return;
    }

    const TrStatusIDs = isCheckStatus.map((status) => {
      return {
        IsSelected: true,
        SelectedID: status.TrStatusID,
      };
    });

    const TrTypeIDs = isCheckTransferType.map((type) => {
      return {
        IsTrSelected: true,
        TrSelectedID: type.TrTypeID,
      };
    });

    let payload;
    payload = {
      FromDate: this.fromDate,
      ToDate: this.toDate,
      CountryType: this.selectedCountryType,
      chkTrxnType: true,
      chkStatus: true,
      AgentID: this.selectedAgent,
      TrStatusIDs,
      TrTypeIDs,
    };
    this.remittancePaidUnPaidService
      .getFillGrid(payload)
      .subscribe((remittance: any) => {

        console.log(remittance)
        if (remittance?.length == 0) {
          alert('No data found.');
          return;
        }else if (remittance.error){
          alert(remittance.error.message);
          return;
        }
        this.remittancePaidUnpaid = remittance;
        this.totalAmount = remittance?.[0]?.TotalAmount;
      });
  }

  get totalNoofTransaction() {
    return this.remittancePaidUnpaid.length;
  }

  get agentName() {
    const agent =
      this.agents.find((a) => a.Agent_ID === this.selectedAgent)?.Agent_Name ||
      '';
    return agent;
  }
}
