
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class AllPaidTransactionsService {
  apiUrl = environment.apiUrl;

  constructor(private http: HttpClient) {}

  private getRequestHeaders(): HttpHeaders {
    return new HttpHeaders({
      Authorization: getCacheData('admin_token'),
      Accept: 'application/json',
      username: getCacheData('username'),
    });
  }

  getCountryList(UserID: number = 0, G_Agent_ID: number = 0, Agent_Country_ID: number = 0) {
    return this.http.get(`${this.apiUrl}/api/master/reports/agent-wise-trxn-paid/country?UserID=${UserID}&G_Agent_ID=${G_Agent_ID}&Agent_Country_ID=${Agent_Country_ID}`, {
      headers: this.getRequestHeaders()
    });
  }

  getAgentList(CountryID: number) {
    return this.http.get(`${this.apiUrl}/api/master/reports/agent-wise-trxn-paid/agent?CountryID=${CountryID}`, {
      headers: this.getRequestHeaders()
    });
  }

  getAgentBranchList(AgentID: number) {
    return this.http.get(`${this.apiUrl}/api/master/reports/agent-wise-trxn-paid/agent-branch?AgentID=${AgentID}`, {
      headers: this.getRequestHeaders()
    });
  }

  getGridData(AgentID: number, CountryID: number, AgentBranchID: number, FromDate: string, ToDate: string) {
    return this.http.get(`${this.apiUrl}/api/master/reports/agent-wise-trxn-paid/grid?AgentID=${AgentID}&CountryID=${CountryID}&AgentBranchID=${AgentBranchID}&FromDate=${FromDate}&ToDate=${ToDate}`, {
      headers: this.getRequestHeaders()
    });
  }



  getTransactionReport(TransactionNumber: any) {
    return this.http.get(
      `${environment.apiUrl}/api/transaction-management/transaction-cancel-paid/transaction-report?TransactionNumber=${TransactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
  
}
