import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class MonthlyFeesReconciliationService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getMonthlyFeesReconciliation(): Observable<any> {
    return this.http.get(`${this.url}/api/master/reports/view-monthly-fees-reconciliation-report/country`, {
      headers: new HttpHeaders({
        Authorization: getCacheData('admin_token'),
        Accept: 'application/json',
        username: getCacheData('username'),
      }),
    });
  }

  getMonthlyFeesReconciliationByAgent(countryID: number): Observable<any> {
    return this.http.get(`${this.url}/api/master/reports/view-monthly-fees-reconciliation-report/agent`, {
      params: { countryID: countryID.toString() },
      headers: new HttpHeaders({
        Authorization: getCacheData('admin_token'),
        Accept: 'application/json',
        username: getCacheData('username'),
      }),
    });
  }

  getMonthlyFeesReconciliationByCurrency(agentID: number): Observable<any> {
    return this.http.get(`${this.url}/api/master/reports/view-monthly-fees-reconciliation-report/currency`, {
      params: { agentID: agentID.toString() },
      headers: new HttpHeaders({
        Authorization: getCacheData('admin_token'),
        Accept: 'application/json',
        username: getCacheData('username'),
      }),
    });
  }

  getMonthlyFeesReconciliationGrid(countryID: number, agentID: number, currencyID: number, fromDate: string, toDate: string): Observable<any> {
    return this.http.get(`${this.url}/api/master/reports/view-monthly-fees-reconciliation-report/grid`, {
      params: {
        CountryID: countryID.toString(),
        AgentID: agentID.toString(),
        CurrencyID: currencyID.toString(),
        FromDate: fromDate,
        ToDate: toDate,
      },
      headers: new HttpHeaders({
        Authorization: getCacheData('admin_token'),
        Accept: 'application/json',
        username: getCacheData('username'),
      }),
    });
  }
}
