import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class StopPaymentService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getStatusList() {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/status-list`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAllStatusList() {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/all-status-list`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentTypeList() {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/agent-type-list`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getSendCountryList() {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/payin-country-list`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getDestinationCountryList() {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/payout-country-list?_MyType=1&_AsPerPermission=true&_CountryID=0`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getSendCurrencyList(countryId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/payin-currency-list?countryId=${countryId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getDestinationCurrencyList(countryId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/payout-currency-list?countryId=${countryId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentList(targetCountryId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/agent-list?_TargetCountryID=${targetCountryId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getUserList(agentId: any = 0, agentBranchId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/agent-user-list?AgentID=${agentId}&Agent_Branch_ID=${agentBranchId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentBranchList(agentId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/agent-branch-list?agentID=${agentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getTransferTypeList() {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/transfer-type-list`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAllData(payload: any) {
    return this.http.post(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/all-data`,
      payload,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  patchApprovedTransaction(payload: any) {
    return this.http.patch(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/approved-transaction`,
      payload,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  displayDetails(appId: any = '', branchId: any = '', refNo: any = '') {
    return this.http.get(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/display-details?AppID=${appId}&Branch=${branchId}&RefNo=${refNo}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  updateStatus(payload: any) {
    return this.http.post(
      `${environment.apiUrl}/api/master/transection-managment/view-stop-payment/update-status`,
      payload,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
}
