import { Component, ElementRef, TemplateRef, ViewChild } from '@angular/core';
import { ViewDuplicateTransactionsService } from '../services/view-duplicate-transactions.service';
import * as XLSX from 'xlsx';
import { AuthService } from '../services/auth.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ViewTransactionStatusForAgentService } from '../view-transaction-status-for-agent.service';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-view-duplicate-transactions',
  templateUrl: './view-duplicate-transactions.component.html',
  styleUrls: ['./view-duplicate-transactions.component.css'],
})
export class ViewDuplicateTransactionsComponent {
  @ViewChild('viewDuplicateTransactionTable', { static: false })
  viewDuplicateTransactionTable: ElementRef;
  @ViewChild('viewTransactionReceipt') viewTransactionReceiptModal;
  bsTransactionReceiptModalRef: BsModalRef;

  fromDate: any = '';
  toDate: any = '';
  transactionNumber: any = '';
  allDuplicateTransactions: any = [];
  selectedFileFormat: any = '1';
  isLoading: boolean = false;
  receiptData: any = {};

  constructor(
    private viewDuplicateTransactionSvc: ViewDuplicateTransactionsService,
    private authSvc: AuthService,
    private ViewTransactionStatusForAgentSvc: ViewTransactionStatusForAgentService,
    private modalService: BsModalService,
    private toastrSvc: ToastrService
  ) { }

  ngOnInit(): void {
    this.fromDate = this.authSvc.getTodayDate();
    this.toDate = this.authSvc.getTodayDate();
  }

  fetchRecords() {
    if (!this.fromDate || !this.toDate) {
      alert('Please Select From & To Date!');
      return;
    }
    this.isLoading = true;

    let observable;

    if (!this.transactionNumber) {
      observable = this.viewDuplicateTransactionSvc.getAllRecord(
        this.fromDate,
        this.toDate
      );
    } else {
      observable = this.viewDuplicateTransactionSvc.getAllRecord(
        this.fromDate,
        this.toDate,
        this.transactionNumber
      );
    }

    observable.subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allDuplicateTransactions = res;
        if (this.allDuplicateTransactions?.length) {
          this.allDuplicateTransactions.forEach((item) => {
            if (typeof item === 'object' && item !== null) {
              item.Status = this.sanitizeHTML(item.Status);
            }
          });
        }
      },
      error: () => {
        this.allDuplicateTransactions = [];
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  displayByDate() {
    this.fetchRecords();
  }

  filterByTransactionNo() {
    if (!this.transactionNumber) {
      alert('Please Fill Transaction Number!');
      return;
    }

    this.fetchRecords();
  }

  sanitizeHTML(htmlString: string): string {
    const doc = new DOMParser().parseFromString(htmlString, 'text/html');
    return doc.body.textContent || '';
  }

  releaseTransaction(transactionNo, genuineTransactionNo) {
    if (transactionNo && genuineTransactionNo) {
      this.viewDuplicateTransactionSvc
        .releaseTransaction(transactionNo, genuineTransactionNo)
        .subscribe({
          next: (res: any) => { },
          error: (err) => {
            // alert(err?.error?.message || 'Something went wrong');
            this.toastrSvc.error(err?.error?.message);
            this.isLoading = false;
          },
          complete: () => {
            this.isLoading = false;
          },
        });
    }
  }

  exportToExcel() {
    console.log(this.viewDuplicateTransactionTable); // Check if the element is defined
    if (!this.viewDuplicateTransactionTable) {
      console.error('Table element not found!');
      return;
    }

    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.viewDuplicateTransactionTable.nativeElement
    );

    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    const fileName = `View_Duplicate_Transactions${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  refresh() {
    this.fetchRecords();
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  getTransactionNumberDetail(item) {
    const trnNo = item?.MasterTxn;
    this.ViewTransactionStatusForAgentSvc.getTransactionReport(trnNo).subscribe(
      (res: any) => {
        if (res?.[0]?.IsDuplicate == true) {
          alert('No data found.');
          return;
        }
        this.receiptData = res?.[0];
        this.openTransactionReceiptModal(this.viewTransactionReceiptModal);
      }
    );
  }

  openTransactionReceiptModal(template: TemplateRef<any>) {
    this.bsTransactionReceiptModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }

  isDataTableBlank(): boolean {
    return (
      !this.allDuplicateTransactions ||
      this.allDuplicateTransactions.length === 0
    );
  }
}
